package MyKeyword;

use strict;
use warnings;

sub class {}

sub field {}

sub method {}

sub isa {}

sub reader {}

sub common {}

class(1);

field(1);

method(3);

isa(4);

reader(5);

common(6);

1;

